package eu.jonahbauer.wizard.client.libgdx.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.utils.I18NBundle;
import com.badlogic.gdx.utils.viewport.ExtendViewport;
import com.badlogic.gdx.utils.viewport.FitViewport;
import eu.jonahbauer.wizard.client.libgdx.MenuAtlas;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.listeners.AutoFocusListener;
import eu.jonahbauer.wizard.client.libgdx.listeners.ButtonKeyListener;
import eu.jonahbauer.wizard.client.libgdx.util.WizardAssetManager;

public abstract class MenuScreen implements Screen {
    protected final float BUTTON_BAR_Y = WizardGame.HEIGHT * 0.15f;

    protected final WizardGame game;
    protected final WizardAssetManager assets;
    protected final SpriteBatch batch;

    protected Stage stage;
    protected FitViewport fitViewport;
    protected ExtendViewport extendViewport;

    // assets
    protected Skin skin;
    protected TextureAtlas atlas;
    protected Sound sfxClick;
    protected I18NBundle messages;

    // textures
    protected TextureRegion background;
    protected TextureRegion title;
    protected TextureRegion[] corners;

    public MenuScreen(WizardGame game) {
        this.game = game;
        this.assets = game.assets;
        this.batch = game.batch;
    }

    @Override
    public void show() {
        assets.loadMenu();
        assets.finishLoading();

        skin = assets.get(WizardAssetManager.SKIN);
        atlas = assets.get(WizardAssetManager.ATLAS_MENU);
        sfxClick = assets.get(WizardAssetManager.SFX_CLICK);
        messages = assets.get(WizardAssetManager.MESSAGES);

        skin.getAll(BitmapFont.class).forEach(entry -> {
            entry.value.getRegion().getTexture().setFilter(Texture.TextureFilter.Linear, Texture.TextureFilter.Linear);
        });

        extendViewport = new ExtendViewport(WizardGame.WIDTH, WizardGame.HEIGHT);
        fitViewport = new FitViewport(WizardGame.WIDTH, WizardGame.HEIGHT);

        stage = new Stage(fitViewport);
        stage.addListener(new ButtonKeyListener());
        stage.addListener(new AutoFocusListener());

        background = atlas.findRegion(MenuAtlas.BACKGROUND);
        title = atlas.findRegion(MenuAtlas.TITLE);
        corners = new TextureRegion[] {
                atlas.findRegion(MenuAtlas.CORNER_TOP_LEFT),
                atlas.findRegion(MenuAtlas.CORNER_BOTTOM_LEFT),
                atlas.findRegion(MenuAtlas.CORNER_BOTTOM_RIGHT),
                atlas.findRegion(MenuAtlas.CORNER_TOP_RIGHT),
        };

        Gdx.input.setInputProcessor(stage);
    }

    @Override
    public final void render(float delta) {
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT | (Gdx.graphics.getBufferFormat().coverageSampling?GL20.GL_COVERAGE_BUFFER_BIT_NV:0));

        extendViewport.apply(true);
        batch.setProjectionMatrix(extendViewport.getCamera().combined);
        batch.begin();
        renderBackground(delta);
        batch.end();

        fitViewport.apply();
        batch.setProjectionMatrix(fitViewport.getCamera().combined);
        batch.begin();
        renderForeground(delta);
        batch.end();

        stage.act(delta);
        stage.draw();
    }

    protected void renderBackground(float delta) {
        batch.setColor(1, 1, 1, 1);
        float scale = Math.max(
                extendViewport.getWorldWidth() / WizardGame.WIDTH,
                extendViewport.getWorldHeight() / WizardGame.HEIGHT
        );
        batch.draw(background, 0, 0, scale * WizardGame.WIDTH, scale * WizardGame.HEIGHT);
        batch.draw(corners[0], 0, extendViewport.getWorldHeight() - corners[0].getRegionHeight());
        batch.draw(corners[1], 0, 0);
        batch.draw(corners[2], extendViewport.getWorldWidth() - corners[2].getRegionWidth(), 0);
        batch.draw(corners[3], extendViewport.getWorldWidth() - corners[3].getRegionWidth(), extendViewport.getWorldHeight() - corners[3].getRegionHeight());
    }

    protected void renderForeground(float delta) {
        batch.setColor(1, 1, 1, 1);
        batch.draw(title, 555, WizardGame.HEIGHT - 192 - 96, 810, 192);
    }

    @Override
    public final void resize(int width, int height) {
        extendViewport.update(width, height);
        fitViewport.update(width, height);
    }

    @Override
    public void pause() {}

    @Override
    public void resume() {}

    @Override
    public void hide() {}

    @Override
    public void dispose() {
        assets.unloadMenu();
        stage.dispose();
    }

    protected void sfxClick() {
        sfxClick.play(0.6f);
    }
}
