package eu.jonahbauer.wizard.client.libgdx.util;

import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.assets.loaders.I18NBundleLoader;
import com.badlogic.gdx.assets.loaders.SkinLoader;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.utils.I18NBundle;
import eu.jonahbauer.wizard.client.libgdx.GameAtlas;
import eu.jonahbauer.wizard.client.libgdx.MenuAtlas;
import eu.jonahbauer.wizard.client.libgdx.UiskinAtlas;
import lombok.experimental.Delegate;

import java.util.Locale;

public class WizardAssetManager {
    public static final String SKIN = "uiskin.json";
    public static final String ATLAS_SKIN = UiskinAtlas.$PATH;
    public static final String ATLAS_GAME = GameAtlas.$PATH;
    public static final String ATLAS_MENU = MenuAtlas.$PATH;

    public static final String SFX_CLICK = "button_click_s.mp3";
    public static final String MUSIC_BACKGROUND = "background.mp3";

    public static final String CURSOR = "cursor.png";
    public static final String MESSAGES = "i18n/messages";


    @Delegate
    private final AssetManager manager = new AssetManager();

    public void loadShared() {
        manager.load(MUSIC_BACKGROUND, Music.class);
        manager.load(CURSOR, Pixmap.class);
        manager.load(MESSAGES, I18NBundle.class, new I18NBundleLoader.I18NBundleParameter(Locale.getDefault()));
    }

    public void loadMenu() {
        manager.load(ATLAS_SKIN, TextureAtlas.class);
        manager.load(ATLAS_MENU, TextureAtlas.class);
        manager.load(SKIN, Skin.class, new SkinLoader.SkinParameter(ATLAS_SKIN));
        manager.load(SFX_CLICK, Sound.class);
    }

    public void unloadMenu() {
        manager.unload(ATLAS_SKIN);
        manager.unload(ATLAS_MENU);
        manager.unload(SKIN);
        manager.unload(SFX_CLICK);
    }

    public void loadGame() {
        manager.load(ATLAS_GAME, TextureAtlas.class);
    }

    public void unloadGame() {
        manager.unload(ATLAS_GAME);
    }
}
